@@ -12,8 +12,10 @@ from typing import (
 )
 
 import pystac
+import pystac.utils
 import pystac.validation
 from pystac import CatalogType, Collection
+from requests import Request
 
 from pystac_client._utils import Modifiable, call_modifier
 from pystac_client.collection_client import CollectionClient
@@ -93,6 +95,8 @@ class Client(pystac.Catalog):
         parameters: Optional[Dict[str, Any]] = None,
         ignore_conformance: bool = False,
         modifier: Optional[Callable[[Modifiable], None]] = None,
+        request_modifier: Optional[Callable[[Request], Union[Request, None]]] = None,
+        stac_io: Optional[StacApiIO] = None,
     ) -> "Client":
         """Opens a STAC Catalog or API
         This function will read the root catalog of a STAC Catalog or API
@@ -128,12 +132,31 @@ class Client(pystac.Catalog):
                 After getting a child collection with, e.g.
                 :meth:`Client.get_collection`, the child items of that collection
                 will still be signed with ``modifier``.
+            request_modifier: A callable that either modifies a `Request` instance or
+                returns a new one. This can be useful for injecting Authentication
+                headers and/or signing fully-formed requests (e.g. signing requests
+                using AWS SigV4).
+
+                The callable should expect a single argument, which will be an instance
+                of :class:`requests.Request`.
+
+                If the callable returns a `requests.Request`, that will be used.
+                Alternately, the callable may simply modify the provided request object
+                and return `None`.
+            stac_io: A `StacApiIO` object to use for I/O requests. Generally, leave
+                this to the default. However in cases where customized I/O processing
+                is required, a custom instance can be provided here.
 
         Return:
             catalog : A :class:`Client` instance for this Catalog/API
         """
         client: Client = cls.from_file(
-            url, headers=headers, parameters=parameters, modifier=modifier
+            url,
+            headers=headers,
+            parameters=parameters,
+            modifier=modifier,
+            request_modifier=request_modifier,
+            stac_io=stac_io,
         )
         search_link = client.get_search_link()
         # if there is a search link, but no conformsTo advertised, ignore
@@ -161,6 +184,7 @@ class Client(pystac.Catalog):
         headers: Optional[Dict[str, str]] = None,
         parameters: Optional[Dict[str, Any]] = None,
         modifier: Optional[Callable[[Modifiable], None]] = None,
+        request_modifier: Optional[Callable[[Request], Union[Request, None]]] = None,
     ) -> "Client":
         """Open a STAC Catalog/API
 
@@ -168,7 +192,11 @@ class Client(pystac.Catalog):
             Client: A Client (PySTAC Catalog) of the root Catalog for this Catalog/API
         """
         if stac_io is None:
-            stac_io = StacApiIO(headers=headers, parameters=parameters)
+            stac_io = StacApiIO(
+                headers=headers,
+                parameters=parameters,
+                request_modifier=request_modifier,
+            )
 
         client: Client = super().from_file(href, stac_io)  # type: ignore
 
@@ -226,7 +254,7 @@ class Client(pystac.Catalog):
             CollectionClient: A STAC Collection
         """
         if self._supports_collections() and self._stac_io:
-            url = f"{self.get_self_href()}/collections/{collection_id}"
+            url = self._get_collections_href(collection_id)
             collection = CollectionClient.from_dict(
                 self._stac_io.read_json(url),
                 root=self,
@@ -253,9 +281,9 @@ class Client(pystac.Catalog):
         """
         collection: Union[Collection, CollectionClient]
 
-        if self._supports_collections() and self.get_self_href() is not None:
-            url = f"{self.get_self_href()}/collections"
-            for page in self._stac_io.get_pages(url):  # type: ignore
+        if self._supports_collections() and self._stac_io:
+            url = self._get_collections_href()
+            for page in self._stac_io.get_pages(url):
                 if "collections" not in page:
                     raise APIError("Invalid response from /collections")
                 for col in page["collections"]:
@@ -476,3 +504,36 @@ class Client(pystac.Catalog):
             ),
             None,
         )
+
+    def _get_collections_href(self, collection_id: Optional[str] = None) -> str:
+        self_href = self.get_self_href()
+        if self_href is None:
+            data_link = self.get_single_link("data")
+            if data_link is None:
+                raise ValueError(
+                    "cannot build a collections href without a self href or a data link"
+                )
+            else:
+                collections_href = data_link.href
+        else:
+            collections_href = f"{self_href.rstrip('/')}/collections"
+
+        if not pystac.utils.is_absolute_href(collections_href):
+            collections_href = self._make_absolute_href(collections_href)
+
+        if collection_id is None:
+            return collections_href
+        else:
+            return f"{collections_href.rstrip('/')}/{collection_id}"
+
+    def _make_absolute_href(self, href: str) -> str:
+        self_link = self.get_single_link("self")
+        if self_link is None:
+            raise ValueError("cannot build an absolute href without a self link")
+        elif not pystac.utils.is_absolute_href(self_link.href):
+            raise ValueError(
+                "cannot build an absolute href from "
+                f"a relative self link: {self_link.href}"
+            )
+        else:
+            return pystac.utils.make_absolute_href(href, self_link.href)
