@@ -16,6 +16,7 @@ from typing import (
     Iterator,
     List,
     Optional,
+    Protocol,
     Tuple,
     Union,
 )
@@ -33,15 +34,17 @@ if TYPE_CHECKING:
     from pystac_client import client as _client
 
 DATETIME_REGEX = re.compile(
-    r"(?P<year>\d{4})(-(?P<month>\d{2})(-(?P<day>\d{2})"
+    r"^(?P<year>\d{4})(-(?P<month>\d{2})(-(?P<day>\d{2})"
     r"(?P<remainder>([Tt])\d{2}:\d{2}:\d{2}(\.\d+)?"
-    r"(?P<tz_info>Z|([-+])(\d{2}):(\d{2}))?)?)?)?"
+    r"(?P<tz_info>[Zz]|([-+])(\d{2}):(\d{2}))?)?)?)?$"
 )
 
-# todo: add runtime_checkable when we drop 3.7 support
-# class GeoInterface(Protocol):
-#     def __geo_interface__(self) -> dict:
-#         ...
+
+class GeoInterface(Protocol):
+    @property
+    def __geo_interface__(self) -> Dict[str, Any]:
+        ...
+
 
 DatetimeOrTimestamp = Optional[Union[datetime_, str]]
 Datetime = str
@@ -62,8 +65,7 @@ IDs = Tuple[str, ...]
 IDsLike = Union[IDs, str, List[str], Iterator[str]]
 
 Intersects = Dict[str, Any]
-IntersectsLike = Union[str, object, Intersects]
-# todo: after 3.7 is dropped, replace object with GeoInterface
+IntersectsLike = Union[str, GeoInterface, Intersects]
 
 Query = Dict[str, Any]
 QueryLike = Union[Query, List[str]]
@@ -437,7 +439,7 @@ class ItemSearch:
     def _to_isoformat_range(
         self,
         component: DatetimeOrTimestamp,
-    ) -> Tuple[Optional[str], Optional[str]]:
+    ) -> Tuple[str, Optional[str]]:
         """Converts a single DatetimeOrTimestamp into one or two Datetimes.
 
         This is required to expand a single value like "2017" out to the whole
@@ -454,6 +456,8 @@ class ItemSearch:
         elif isinstance(component, str):
             if component == "..":
                 return component, None
+            elif component == "":
+                return "..", None
 
             match = DATETIME_REGEX.match(component)
             if not match:
@@ -502,12 +506,16 @@ class ItemSearch:
         if not components:
             return None
         elif len(components) == 1:
+            if components[0] is None:
+                raise Exception("cannot create a datetime query with None")
             start, end = self._to_isoformat_range(components[0])
             if end is not None:
                 return f"{start}/{end}"
             else:
                 return start
         elif len(components) == 2:
+            if all(c is None for c in components):
+                raise Exception("cannot create a double open-ended interval")
             start, _ = self._to_isoformat_range(components[0])
             backup_end, end = self._to_isoformat_range(components[1])
             return f"{start}/{end or backup_end}"
